<?php
/*
Plugin Name: AWS CloudFront CDN Support with Cache Purging, Asset Selection & Enhancements
Description: Integrates AWS CloudFront with WordPress. Choose which assets to serve via CloudFront, purge cache on demand, exclude wp-admin pages from CDN rewriting, and optionally use custom invalidation paths.
Version: 1.3
Author: Your Name
*/

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly.
}

// If the AWS SDK for PHP is not already loaded via Composer/autoload, include it here.
// Example: require_once __DIR__ . '/vendor/autoload.php';

/*---------------------------------
  Admin Settings Page
----------------------------------*/

/**
 * Add the CloudFront settings page under Settings.
 */
function cfc_plugin_menu() {
    add_options_page(
        'CloudFront CDN Settings',    // Page title.
        'CloudFront CDN',             // Menu title.
        'manage_options',             // Capability.
        'cloudfront-cdn-settings',    // Menu slug.
        'cfc_plugin_settings_page'    // Callback function.
    );
}
add_action('admin_menu', 'cfc_plugin_menu');

/**
 * Register plugin settings.
 */
function cfc_register_settings() {
    register_setting('cfc-settings-group', 'cfc_cloudfront_domain');
    register_setting('cfc-settings-group', 'cfc_aws_access_key');
    register_setting('cfc-settings-group', 'cfc_aws_secret_key');
    register_setting('cfc-settings-group', 'cfc_distribution_id');
    register_setting('cfc-settings-group', 'cfc_invalidation_paths'); // New setting for custom invalidation paths.
    register_setting('cfc-settings-group', 'cfc_serve_images');
    register_setting('cfc-settings-group', 'cfc_serve_scripts');
    register_setting('cfc-settings-group', 'cfc_serve_styles');
    register_setting('cfc-settings-group', 'cfc_serve_plugins');
}
add_action('admin_init', 'cfc_register_settings');

/**
 * Display the settings page.
 */
function cfc_plugin_settings_page() {
    ?>
    <div class="wrap">
        <h1>CloudFront CDN Settings</h1>
        <form method="post" action="options.php">
            <?php settings_fields('cfc-settings-group'); ?>
            <?php do_settings_sections('cfc-settings-group'); ?>
            <table class="form-table">
                <tr valign="top">
                    <th scope="row">CloudFront Domain</th>
                    <td>
                        <input type="text" name="cfc_cloudfront_domain" value="<?php echo esc_attr(get_option('cfc_cloudfront_domain')); ?>" placeholder="d1234567890.cloudfront.net" />
                        <p class="description">Enter your CloudFront distribution domain (without http:// or https://).</p>
                    </td>
                </tr>
                <tr valign="top">
                    <th scope="row">AWS Access Key ID</th>
                    <td>
                        <input type="text" name="cfc_aws_access_key" value="<?php echo esc_attr(get_option('cfc_aws_access_key')); ?>" />
                    </td>
                </tr>
                <tr valign="top">
                    <th scope="row">AWS Secret Access Key</th>
                    <td>
                        <input type="password" name="cfc_aws_secret_key" value="<?php echo esc_attr(get_option('cfc_aws_secret_key')); ?>" />
                    </td>
                </tr>
                <tr valign="top">
                    <th scope="row">CloudFront Distribution ID</th>
                    <td>
                        <input type="text" name="cfc_distribution_id" value="<?php echo esc_attr(get_option('cfc_distribution_id')); ?>" placeholder="E1234567890" />
                    </td>
                </tr>
                <tr valign="top">
                    <th scope="row">Custom Invalidation Paths</th>
                    <td>
                        <input type="text" name="cfc_invalidation_paths" value="<?php echo esc_attr(get_option('cfc_invalidation_paths')); ?>" placeholder="e.g., /*, /css/*, /js/*" />
                        <p class="description">Enter comma-separated paths to invalidate. Leave blank to default to "/*".</p>
                    </td>
                </tr>
                <tr valign="top">
                    <th scope="row">Assets to Serve via CloudFront</th>
                    <td>
                        <label>
                            <input type="checkbox" name="cfc_serve_images" value="1" <?php checked(1, get_option('cfc_serve_images'), true); ?> />
                            Images
                        </label><br/>
                        <label>
                            <input type="checkbox" name="cfc_serve_scripts" value="1" <?php checked(1, get_option('cfc_serve_scripts'), true); ?> />
                            Scripts
                        </label><br/>
                        <label>
                            <input type="checkbox" name="cfc_serve_styles" value="1" <?php checked(1, get_option('cfc_serve_styles'), true); ?> />
                            Styles
                        </label><br/>
                        <label>
                            <input type="checkbox" name="cfc_serve_plugins" value="1" <?php checked(1, get_option('cfc_serve_plugins'), true); ?> />
                            Plugins
                        </label>
                    </td>
                </tr>
            </table>
            <?php submit_button('Save Changes'); ?>
        </form>
        <hr />
        <h2>Cache Purging</h2>
        <form method="post" action="<?php echo admin_url('admin-post.php'); ?>">
            <input type="hidden" name="action" value="cfc_purge_cache">
            <?php 
            // Use a custom nonce field name to avoid duplicate IDs.
            wp_nonce_field('cfc_purge_cache_nonce', 'cfc_purge_nonce'); 
            ?>
            <?php 
            // Provide a custom ID for the submit button to avoid duplicate IDs.
            submit_button('Purge CloudFront Cache', 'primary', 'submit_purge', false, array('id' => 'purge_submit')); 
            ?>
        </form>
    </div>
    <?php
}

/*---------------------------------
  URL Rewriting Functionality
----------------------------------*/

/**
 * Replace URLs for static assets with the CloudFront domain.
 *
 * @param string $url Original URL.
 * @return string Modified URL.
 */
function cfc_replace_urls_with_cloudfront($url) {
    // Do not rewrite URLs in the admin area.
    if ( is_admin() ) {
        return $url;
    }
    
    static $home_url = null;
    if ( is_null($home_url) ) {
        $home_url = home_url();
    }
    
    $cloudfront_domain = get_option('cfc_cloudfront_domain');
    
    // If no CloudFront domain is set, return the original URL.
    if ( empty( $cloudfront_domain ) ) {
        return $url;
    }
    
    // Only modify URLs that point to the current site.
    if ( strpos( $url, $home_url ) !== false ) {
        $parsed_home = parse_url($home_url);
        $parsed_url  = parse_url($url);
        if ( isset($parsed_home['host']) && isset($parsed_url['host']) ) {
            return str_replace($parsed_home['host'], $cloudfront_domain, $url);
        }
    }
    return $url;
}

// Apply URL rewriting conditionally based on asset selection.
if ( get_option('cfc_serve_images') ) {
    add_filter('wp_get_attachment_url', 'cfc_replace_urls_with_cloudfront');
}
if ( get_option('cfc_serve_plugins') ) {
    add_filter('plugins_url', 'cfc_replace_urls_with_cloudfront');
}
if ( get_option('cfc_serve_scripts') ) {
    add_filter('script_loader_src', 'cfc_replace_urls_with_cloudfront');
}
if ( get_option('cfc_serve_styles') ) {
    add_filter('style_loader_src', 'cfc_replace_urls_with_cloudfront');
}

/*---------------------------------
  CORS Header Functionality
----------------------------------*/

/**
 * Add CORS header on frontend pages to help avoid opaque response blocking.
 */
function cfc_add_cors_headers() {
    // Only add the header on frontend pages.
    if ( ! is_admin() ) {
        header("Access-Control-Allow-Origin: *");
    }
}
add_action('send_headers', 'cfc_add_cors_headers');

/*---------------------------------
  Cache Purging Functionality
----------------------------------*/

/**
 * Purge CloudFront cache by creating an invalidation.
 *
 * @return mixed Result of the invalidation or a WP_Error on failure.
 */
function cfc_purge_cloudfront_cache() {
    $access_key      = get_option('cfc_aws_access_key');
    $secret_key      = get_option('cfc_aws_secret_key');
    $distribution_id = get_option('cfc_distribution_id');
    
    if ( empty($access_key) || empty($secret_key) || empty($distribution_id) ) {
        return new WP_Error('missing_credentials', 'AWS credentials or distribution ID not set.');
    }
    
    if ( ! class_exists('Aws\CloudFront\CloudFrontClient') ) {
        return new WP_Error('aws_sdk_missing', 'AWS SDK for PHP is not available.');
    }
    
    // Determine invalidation paths.
    $paths_setting = get_option('cfc_invalidation_paths');
    if ( empty(trim($paths_setting)) ) {
        $paths = array('/*');
    } else {
        $paths = array_map('trim', explode(',', $paths_setting));
    }
    
    try {
        $client = new Aws\CloudFront\CloudFrontClient([
            'version'     => 'latest',
            'region'      => 'us-east-1', // Adjust the region as needed.
            'credentials' => [
                'key'    => $access_key,
                'secret' => $secret_key,
            ],
        ]);
        
        $result = $client->createInvalidation([
            'DistributionId'    => $distribution_id,
            'InvalidationBatch' => [
                'Paths' => [
                    'Quantity' => count($paths),
                    'Items'    => $paths,
                ],
                'CallerReference' => time(),
            ],
        ]);
        
        return $result;
    } catch ( Exception $e ) {
        return new WP_Error('invalidation_error', $e->getMessage());
    }
}

/**
 * Handle the cache purge request from the settings page.
 */
function cfc_handle_purge_cache() {
    if ( ! current_user_can('manage_options') ) {
        wp_die('Unauthorized user');
    }
    
    // Check the custom nonce field.
    check_admin_referer('cfc_purge_cache_nonce', 'cfc_purge_nonce');
    
    $result = cfc_purge_cloudfront_cache();
    
    if ( is_wp_error($result) ) {
        $message = 'Cache purging failed: ' . $result->get_error_message();
    } else {
        $message = 'CloudFront cache purged successfully.';
    }
    
    wp_redirect(add_query_arg('cfc_message', urlencode($message), wp_get_referer()));
    exit;
}
add_action('admin_post_cfc_purge_cache', 'cfc_handle_purge_cache');
